# Get content from the config.json
$config = Get-Content -Path .\config.json | ConvertFrom-Json

$xu_location = $config.xu_location
$xu_server = $config.xu_server
$xu_port = $config.xu_port
$end_of_upload_api = $config.end_of_upload_api

$extraction_start_date = $config.extraction_start_date
$extraction_end_date = $config.extraction_end_date
$language = $config.language
$exchange_rate_type = $config.exchange_rate_type

# Create a new log file for every run
$logFile = ".\extraction.log"

if (Test-Path -Path .\$logFile -PathType Leaf) {
    Remove-Item .\$logFile
}

# Function for appending text to the log file
function WriteLog
{
Param ([string]$logText)
$timestamp = (Get-Date).toString("dd/MM/yyyy HH:mm:ss")
Add-content $logFile -value "$timestamp $logText"
Write-Output $logText
}

# Function for checking the error code of extractions
function CheckExtractionError
{
Param ([string]$extraction)
if ($LASTEXITCODE -ne 0) {
    WriteLog("$extraction failed with return code: $LASTEXITCODE")
    Set-Variable -Name "extraction_failed" -Value 1 -Scope global
}
}

# If at least one extraction fails, do not upload to blob storage.
Set-Variable -Name "extraction_failed" -Value 0 -Scope global

# Extraction commands
# Transactional tables
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_BKPF" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_BKPF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_BSAK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_BSAK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_BSE_CLR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_BSE_CLR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_BSIK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_BSIK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_CDHDR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_CDHDR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_CDPOS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_CDPOS')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_EBAN" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_EBAN')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_EKBE" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_EKBE')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_EKET"
CheckExtractionError('AP_EKET')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_EKKN"
CheckExtractionError('AP_EKKN')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_EKKO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_EKKO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_EKPO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_EKPO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_RBKP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_RBKP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_RSEG" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('AP_RSEG')

# Master data tables 
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_ADCP"
CheckExtractionError('AP_ADCP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_ADRP"
CheckExtractionError('AP_ADRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_CSKT" -o "language=$language"
CheckExtractionError('AP_CSKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_KNA1"
CheckExtractionError('AP_KNA1')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_LFA1"
CheckExtractionError('AP_LFA1')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_MAKT" -o "language=$language"
CheckExtractionError('AP_MAKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T001"
CheckExtractionError('AP_T001')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T001L"
CheckExtractionError('AP_T001L')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T001W"
CheckExtractionError('AP_T001W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T003T" -o "language=$language"
CheckExtractionError('AP_T003T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T005T" -o "language=$language"
CheckExtractionError('AP_T005T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T005U" -o "language=$language"
CheckExtractionError('AP_T005U')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T006A" -o "language=$language"
CheckExtractionError('AP_T006A')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T008T" -o "language=$language"
CheckExtractionError('AP_T008T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T023T" -o "language=$language"
CheckExtractionError('AP_T023T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T024"
CheckExtractionError('AP_T024')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T024E"
CheckExtractionError('AP_T024E')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T052U" -o "language=$language"
CheckExtractionError('AP_T052U')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_T161T" -o "language=$language"
CheckExtractionError('AP_T161T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_TCURF" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('AP_TCURF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_TCURR" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('AP_TCURR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_TCURX"
CheckExtractionError('AP_TCURX')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_TGSBT" -o "language=$language"
CheckExtractionError('AP_TGSBT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_USR02"
CheckExtractionError('AP_USR02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "AP_USR21"
CheckExtractionError('AP_USR21')

# Only ingest data if extraction is successful.
if ($extraction_failed) {
    Exit
} else {
    Invoke-WebRequest -Method POST -Uri $end_of_upload_api
}
